﻿using System;
using System.Security;
using System.Security.Permissions;
using System.Text;

namespace Mindsharp.Utilities
{
  [Serializable()]
  public sealed class PrivilegedSandboxPermission :
    System.Security.CodeAccessPermission,
    System.Security.Permissions.IUnrestrictedPermission
  {
    private bool unrestricted;

    // ctor to set the initial state
    public PrivilegedSandboxPermission(PermissionState state)
    {
      if (PermissionState.Unrestricted == state)
      {
        unrestricted = true;
      }
      else
      {
        unrestricted = false;
      }
    }

    public bool IsUnrestricted()
    {
      return unrestricted;
    }

    // Return new object with current value of unrestricted
    public override IPermission Copy()
    {
      PrivilegedSandboxPermission copy =
        new PrivilegedSandboxPermission(PermissionState.None);

      copy.unrestricted = IsUnrestricted();

      return copy;
    }

    public override IPermission Intersect(IPermission target)
    {
      if (null == target) return null;

      try
      {
        // Create a new instance of PrivilegedSandboxPermission from the passed object
        PrivilegedSandboxPermission PassedPermission =
          (PrivilegedSandboxPermission)target;

        if (PassedPermission.unrestricted)
        {
          // Return a copy of the current class if the passed perm is also unrestricted
          return Copy();
        }
        else
        {
          // Return the target restricted target
          return target;
        }
      }
      // Throw InvalidCastException ArgumentException to notify the developer
      catch (InvalidCastException)
      {
        throw new ArgumentException("Argument_WrongType", GetType().FullName);
      }
    }

    public override bool IsSubsetOf(IPermission target)
    {
      // If nothing was passed return the opposite of unrestricted
      if (null == target) return !unrestricted;

      try
      {
        // Create a new instance of PrivilegedSandboxPermission from the passed object
        PrivilegedSandboxPermission PassedPermission =
          (PrivilegedSandboxPermission)target;

        // If unrestricted has the same value in both objects
        // then one is the subset of the other
        if (unrestricted.Equals(PassedPermission.unrestricted))
        {
          return true;
        }
        else
        {
          return false;
        }
      }
      // Throw InvalidCastException ArgumentException to notify the developer
      catch (InvalidCastException)
      {
        throw new ArgumentException("Argument_WrongType", GetType().FullName);
      }
    }

    public override void FromXml(SecurityElement PassedElement)
    {
      // Get the unrestricted value from the XML and initialize 
      // the current instance of unrestricted to that value
      string element = PassedElement.Attribute("Unrestricted");

      if (!string.IsNullOrEmpty(element))
      {
        unrestricted = Convert.ToBoolean(element);
      }
    }

    public override SecurityElement ToXml()
    {
      // Encode the current permission to XML using the SecurityElement class
      SecurityElement element = new SecurityElement("IPermission");
      Type type = GetType();

      StringBuilder AssemblyName = new StringBuilder(type.Assembly.ToString());
      AssemblyName.Replace('\"', '\'');
      element.AddAttribute("class", type.FullName + ", " + AssemblyName);
      element.AddAttribute("version", "1");
      element.AddAttribute("Unrestricted", unrestricted.ToString());
      return element;
    }
  }
}